@@ -10,12 +10,12 @@ from statsmodels.compat.python import long
 from warnings import warn
 
 import numpy as np
-import pandas as pd
-from .kalman_filter import KalmanFilter, FilterResults
+from .kalman_filter import KalmanFilter
 from .mlemodel import MLEModel, MLEResults, MLEResultsWrapper
 from .tools import (
     companion_matrix, diff, is_invertible, constrain_stationary_univariate,
-    unconstrain_stationary_univariate, solve_discrete_lyapunov
+    unconstrain_stationary_univariate, solve_discrete_lyapunov,
+    prepare_exog
 )
 from statsmodels.tools.tools import Bunch
 from statsmodels.tools.data import _is_using_pandas
@@ -78,7 +78,7 @@ class SARIMAX(MLEModel):
     simple_differencing : boolean, optional
         Whether or not to use partially conditional maximum likelihood
         estimation. If True, differencing is performed prior to estimation,
-        which discards the first :math:`s D + d` initial rows but reuslts in a
+        which discards the first :math:`s D + d` initial rows but results in a
         smaller state-space formulation. If False, the full SARIMAX model is
         put in state-space form so that all datapoints can be used in
         estimation. Default is False.
@@ -310,18 +310,18 @@ class SARIMAX(MLEModel):
         # Assume that they are given from lowest degree to highest, that all
         # degrees except for the constant are included, and that they are
         # boolean vectors (0 for not included, 1 for included).
-        if isinstance(order[0], (int, long)):
+        if isinstance(order[0], (int, long, np.integer)):
             self.polynomial_ar = np.r_[1., np.ones(order[0])]
         else:
             self.polynomial_ar = np.r_[1., order[0]]
-        if isinstance(order[2], (int, long)):
+        if isinstance(order[2], (int, long, np.integer)):
             self.polynomial_ma = np.r_[1., np.ones(order[2])]
         else:
             self.polynomial_ma = np.r_[1., order[2]]
         # Assume that they are given from lowest degree to highest, that the
         # degrees correspond to (1*s, 2*s, ..., P*s), and that they are
         # boolean vectors (0 for not included, 1 for included).
-        if isinstance(seasonal_order[0], (int, long)):
+        if isinstance(seasonal_order[0], (int, long, np.integer)):
             self.polynomial_seasonal_ar = np.r_[
                 1.,  # constant
                 ([0] * (self.seasonal_periods - 1) + [1]) * seasonal_order[0]
@@ -331,10 +331,9 @@ class SARIMAX(MLEModel):
                 1., [0] * self.seasonal_periods * len(seasonal_order[0])
             ]
             for i in range(len(seasonal_order[0])):
-                self.polynomial_seasonal_ar[(i + 1) * self.seasonal_periods] = (
-                    seasonal_order[0][i]
-                )
-        if isinstance(seasonal_order[2], (int, long)):
+                tmp = (i + 1) * self.seasonal_periods
+                self.polynomial_seasonal_ar[tmp] = seasonal_order[0][i]
+        if isinstance(seasonal_order[2], (int, long, np.integer)):
             self.polynomial_seasonal_ma = np.r_[
                 1.,  # constant
                 ([0] * (self.seasonal_periods - 1) + [1]) * seasonal_order[2]
@@ -344,9 +343,8 @@ class SARIMAX(MLEModel):
                 1., [0] * self.seasonal_periods * len(seasonal_order[2])
             ]
             for i in range(len(seasonal_order[2])):
-                self.polynomial_seasonal_ma[(i + 1) * self.seasonal_periods] = (
-                    seasonal_order[2][i]
-                )
+                tmp = (i + 1) * self.seasonal_periods
+                self.polynomial_seasonal_ma[tmp] = seasonal_order[2][i]
 
         # Deterministic trend polynomial
         self.trend = trend
@@ -409,23 +407,13 @@ class SARIMAX(MLEModel):
         self._k_order = max(self.k_ar + self.k_seasonal_ar,
                             self.k_ma + self.k_seasonal_ma + 1)
         if self._k_order == 1 and self.k_ar + self.k_seasonal_ar == 0:
-            self._k_order = 0
+            # Handle time-varying regression
+            if self.time_varying_regression:
+                self._k_order = 0
 
         # Exogenous data
-        self.k_exog = 0
-        if exog is not None:
-            exog_is_using_pandas = _is_using_pandas(exog, None)
-            if not exog_is_using_pandas:
-                exog = np.asarray(exog)
+        (self.k_exog, exog) = prepare_exog(exog)
 
-            # Make sure we have 2-dimensional array
-            if exog.ndim < 2:
-                if not exog_is_using_pandas:
-                    exog = np.atleast_2d(exog).T
-                else:
-                    exog = pd.DataFrame(exog)
-
-            self.k_exog = exog.shape[1]
         # Redefine mle_regression to be true only if it was previously set to
         # true and there are exogenous regressors
         self.mle_regression = (
@@ -444,7 +432,8 @@ class SARIMAX(MLEModel):
         # Number of states
         k_states = self._k_order
         if not self.simple_differencing:
-            k_states += self.seasonal_periods * self._k_seasonal_diff + self._k_diff
+            k_states += (self.seasonal_periods * self._k_seasonal_diff +
+                         self._k_diff)
         if self.state_regression:
             k_states += self.k_exog
 
@@ -519,10 +508,10 @@ class SARIMAX(MLEModel):
             self._manual_initialization = True
 
         # Initialize the fixed components of the statespace model
-        self.ssm.design = self.initial_design
-        self.ssm.state_intercept = self.initial_state_intercept
-        self.ssm.transition = self.initial_transition
-        self.ssm.selection = self.initial_selection
+        self.ssm['design'] = self.initial_design
+        self.ssm['state_intercept'] = self.initial_state_intercept
+        self.ssm['transition'] = self.initial_transition
+        self.ssm['selection'] = self.initial_selection
 
         # If we are estimating a simple ARMA model, then we can use a faster
         # initialization method (unless initialization was already specified).
@@ -561,14 +550,20 @@ class SARIMAX(MLEModel):
                 exog = diff(exog.copy(), self.orig_k_diff,
                             self.orig_k_seasonal_diff, self.seasonal_periods)
 
-            # Reset the ModelData datasets
+            # Reset the ModelData datasets and cache
             self.data.endog, self.data.exog = (
                 self.data._convert_endog_exog(endog, exog))
 
-            # Reset dates, if provided
-            if self.data.dates is not None:
-                new_length = self.data.endog.shape[0]
-                self.data.dates = self.data.dates[orig_length - new_length:]
+            # Reset indexes, if provided
+            new_length = self.data.endog.shape[0]
+            if self.data.row_labels is not None:
+                self.data._cache['row_labels'] = (
+                    self.data.row_labels[orig_length - new_length:])
+            if self._index is not None:
+                if self._index_generated:
+                    self._index = self._index[:-(orig_length - new_length)]
+                else:
+                    self._index = self._index[orig_length - new_length:]
 
         # Reset the nobs
         self.nobs = endog.shape[0]
@@ -718,20 +713,33 @@ class SARIMAX(MLEModel):
 
         # Add in the initialized stationary components
         if self._k_order > 0:
-            selection_stationary = self.ssm.selection[start:end, :, 0]
+            transition = self.ssm['transition', start:end, start:end, 0]
+
+            # Initial state
+            # In the Harvey representation, if we have a trend that
+            # is put into the state intercept and means we have a non-zero
+            # unconditional mean
+            if not self.hamilton_representation and self.k_trend > 0:
+                initial_intercept = (
+                    self['state_intercept', self._k_states_diff, 0])
+                initial_mean = (initial_intercept /
+                                (1 - np.sum(transition[:, 0])))
+                initial_state[self._k_states_diff] = initial_mean
+                _start = self._k_states_diff + 1
+                _end = _start + transition.shape[0] - 1
+                initial_state[_start:_end] = transition[1:, 0] * initial_mean
+
+            # Initial state covariance
+            selection_stationary = self.ssm['selection', start:end, :, 0]
             selected_state_cov_stationary = np.dot(
-                np.dot(selection_stationary, self.ssm.state_cov[:, :, 0]),
-                selection_stationary.T
-            )
+                np.dot(selection_stationary, self.ssm['state_cov', :, :, 0]),
+                selection_stationary.T)
             initial_state_cov_stationary = solve_discrete_lyapunov(
-                self.ssm.transition[start:end, start:end, 0],
-                selected_state_cov_stationary,
-                complex_step=complex_step
-            )
+                transition, selected_state_cov_stationary,
+                complex_step=complex_step)
 
             initial_state_cov[start:end, start:end] = (
-                initial_state_cov_stationary
-            )
+                initial_state_cov_stationary)
 
         self.ssm.initialize_known(initial_state, initial_state_cov)
 
@@ -745,6 +753,9 @@ class SARIMAX(MLEModel):
             [1] * self.state_error, [0] * (self._k_order - 1)
         ]
 
+        if len(design) == 0:
+            design = np.r_[0]
+
         # If we have exogenous regressors included as part of the state vector
         # then the exogenous data is incorporated as a time-varying component
         # of the design matrix
@@ -792,11 +803,12 @@ class SARIMAX(MLEModel):
             end = None
 
         # T_c
-        transition[start:end, start:end] = companion_matrix(self._k_order)
-        if self.hamilton_representation:
-            transition[start:end, start:end] = np.transpose(
-                companion_matrix(self._k_order)
-            )
+        if self._k_order > 0:
+            transition[start:end, start:end] = companion_matrix(self._k_order)
+            if self.hamilton_representation:
+                transition[start:end, start:end] = np.transpose(
+                    companion_matrix(self._k_order)
+                )
 
         # Seasonal differencing component
         # T^*
@@ -827,8 +839,8 @@ class SARIMAX(MLEModel):
                 start = self._k_diff
                 end = self._k_states_diff
                 transition[:self._k_diff, start:end] = (
-                    ([0] * (self.seasonal_periods - 1) + [1]) * self._k_seasonal_diff
-                )
+                    ([0] * (self.seasonal_periods - 1) + [1]) *
+                    self._k_seasonal_diff)
             # [1 0]
             column = self._k_states_diff
             transition[:self._k_diff, column] = 1
@@ -845,6 +857,9 @@ class SARIMAX(MLEModel):
                     [1] * (self._k_order > 0), [0] * (self._k_order - 1),
                     [0] * ((1 - self.mle_regression) * self.k_exog)
                 ][:, None]
+
+                if len(selection) == 0:
+                    selection = np.zeros((self.k_states, self.k_posdef))
             else:
                 selection = np.zeros((self.k_states, 0))
         else:
@@ -857,15 +872,9 @@ class SARIMAX(MLEModel):
                 selection[-i, -i] = 1
         return selection
 
-    def filter(self, params, **kwargs):
-        kwargs.setdefault('results_class', SARIMAXResults)
-        kwargs.setdefault('results_wrapper_class', SARIMAXResultsWrapper)
-        return super(SARIMAX, self).filter(params, **kwargs)
-
-    def smooth(self, params, **kwargs):
-        kwargs.setdefault('results_class', SARIMAXResults)
-        kwargs.setdefault('results_wrapper_class', SARIMAXResultsWrapper)
-        return super(SARIMAX, self).smooth(params, **kwargs)
+    @property
+    def _res_classes(self):
+        return {'fit': (SARIMAXResults, SARIMAXResultsWrapper)}
 
     @staticmethod
     def _conditional_sum_squares(endog, k_ar, polynomial_ar, k_ma,
@@ -1043,9 +1052,9 @@ class SARIMAX(MLEModel):
             if not params_seasonal_variance == []:
                 params_variance = params_seasonal_variance
             elif self.k_exog > 0:
-                params_variance = np.dot(endog, endog)
+                params_variance = np.inner(endog, endog)
             else:
-                params_variance = 1
+                params_variance = np.inner(endog, endog) / self.nobs
         params_measurement_variance = 1 if self.measurement_error else []
 
         # Combine all parameters
@@ -1327,7 +1336,7 @@ class SARIMAX(MLEModel):
             end += self.k_ma_params
             if self.enforce_invertibility:
                 constrained[start:end] = (
-                    constrain_stationary_univariate(unconstrained[start:end])
+                    -constrain_stationary_univariate(unconstrained[start:end])
                 )
             else:
                 constrained[start:end] = unconstrained[start:end]
@@ -1349,7 +1358,7 @@ class SARIMAX(MLEModel):
             end += self.k_seasonal_ma_params
             if self.enforce_invertibility:
                 constrained[start:end] = (
-                    constrain_stationary_univariate(unconstrained[start:end])
+                    -constrain_stationary_univariate(unconstrained[start:end])
                 )
             else:
                 constrained[start:end] = unconstrained[start:end]
@@ -1431,7 +1440,7 @@ class SARIMAX(MLEModel):
             end += self.k_ma_params
             if self.enforce_invertibility:
                 unconstrained[start:end] = (
-                    unconstrain_stationary_univariate(constrained[start:end])
+                    unconstrain_stationary_univariate(-constrained[start:end])
                 )
             else:
                 unconstrained[start:end] = constrained[start:end]
@@ -1453,7 +1462,7 @@ class SARIMAX(MLEModel):
             end += self.k_seasonal_ma_params
             if self.enforce_invertibility:
                 unconstrained[start:end] = (
-                    unconstrain_stationary_univariate(constrained[start:end])
+                    unconstrain_stationary_univariate(-constrained[start:end])
                 )
             else:
                 unconstrained[start:end] = constrained[start:end]
@@ -1631,7 +1640,7 @@ class SARIMAX(MLEModel):
                     self.ssm.obs_intercept += data[None, :]
                 # Otherwise set it directly
                 else:
-                    self.ssm.obs_intercept = data[None, :]
+                    self.ssm['obs_intercept'] = data[None, :]
 
         # Observation covariance matrix
         if self.measurement_error:
@@ -1644,7 +1653,8 @@ class SARIMAX(MLEModel):
             # This is required if the transition matrix is not really in use
             # (e.g. for an MA(q) process) so that it's dtype never changes as
             # the parameters' dtype changes. This changes the dtype manually.
-            self.ssm.transition = self.ssm.transition.real.astype(params.dtype)
+            self.ssm['transition'] = self.ssm['transition'].real.astype(
+                params.dtype)
 
         # Selection matrix (Harvey) or Design matrix (Hamilton)
         if self.k_ma > 0 or self.k_seasonal_ma > 0:
@@ -1711,7 +1721,8 @@ class SARIMAXResults(MLEResults):
     statsmodels.tsa.statespace.kalman_filter.FilterResults
     statsmodels.tsa.statespace.mlemodel.MLEResults
     """
-    def __init__(self, model, params, filter_results, cov_type='opg', **kwargs):
+    def __init__(self, model, params, filter_results, cov_type='opg',
+                 **kwargs):
         super(SARIMAXResults, self).__init__(model, params, filter_results,
                                              cov_type, **kwargs)
 
@@ -1773,9 +1784,19 @@ class SARIMAXResults(MLEResults):
         self.param_terms = self.model.param_terms
         start = end = 0
         for name in self.param_terms:
-            end += self.model_orders[name]
+            if name == 'ar':
+                k = self.model.k_ar_params
+            elif name == 'ma':
+                k = self.model.k_ma_params
+            elif name == 'seasonal_ar':
+                k = self.model.k_seasonal_ar_params
+            elif name == 'seasonal_ma':
+                k = self.model.k_seasonal_ma_params
+            else:
+                k = self.model_orders[name]
+            end += k
             setattr(self, '_params_%s' % name, self.params[start:end])
-            start += self.model_orders[name]
+            start += k
 
         # Handle removing data
         self._data_attr_model.extend(['orig_endog', 'orig_exog'])
@@ -1820,20 +1841,42 @@ class SARIMAXResults(MLEResults):
     def arparams(self):
         """
         (array) Autoregressive parameters actually estimated in the model.
-        Does not include parameters whose values are constrained to be zero.
+        Does not include seasonal autoregressive parameters (see
+        `seasonalarparams`) or parameters whose values are constrained to be
+        zero.
         """
         return self._params_ar
 
+    @cache_readonly
+    def seasonalarparams(self):
+        """
+        (array) Seasonal autoregressive parameters actually estimated in the
+        model. Does not include nonseasonal autoregressive parameters (see
+        `arparams`) or parameters whose values are constrained to be zero.
+        """
+        return self._params_seasonal_ar
+
     @cache_readonly
     def maparams(self):
         """
         (array) Moving average parameters actually estimated in the model.
-        Does not include parameters whose values are constrained to be zero.
+        Does not include seasonal moving average parameters (see
+        `seasonalmaparams`) or parameters whose values are constrained to be
+        zero.
         """
         return self._params_ma
 
-    def get_prediction(self, start=None, end=None, dynamic=False, exog=None,
-                **kwargs):
+    @cache_readonly
+    def seasonalmaparams(self):
+        """
+        (array) Seasonal moving average parameters actually estimated in the
+        model. Does not include nonseasonal moving average parameters (see
+        `maparams`) or parameters whose values are constrained to be zero.
+        """
+        return self._params_seasonal_ma
+
+    def get_prediction(self, start=None, end=None, dynamic=False, index=None,
+                       exog=None, **kwargs):
         """
         In-sample prediction and out-of-sample forecasting
 
@@ -1876,11 +1919,11 @@ class SARIMAXResults(MLEResults):
             Array of out of sample forecasts.
         """
         if start is None:
-            start = 0
+            start = self.model._index[0]
 
-        # Handle end (e.g. date)
-        _start = self.model._get_predict_start(start)
-        _end, _out_of_sample = self.model._get_predict_end(end)
+        # Handle start, end, dynamic
+        _start, _end, _out_of_sample, prediction_index = (
+            self.model._get_prediction_index(start, end, index, silent=True))
 
         # Handle exogenous parameters
         if _out_of_sample and (self.model.k_exog + self.model.k_trend > 0):
@@ -1924,8 +1967,8 @@ class SARIMAXResults(MLEResults):
                  ' required. `exog` argument ignored.', ValueWarning)
 
         return super(SARIMAXResults, self).get_prediction(
-            start=start, end=end, dynamic=dynamic, exog=exog, **kwargs
-        )
+            start=start, end=end, dynamic=dynamic, index=index, exog=exog,
+            **kwargs)
 
     def summary(self, alpha=.05, start=None):
         # Create the model name
@@ -1976,7 +2019,8 @@ class SARIMAXResults(MLEResults):
                 k_seasonal_diff = 0
             seasonal_order = ('(%s, %d, %s, %d)' %
                               (str(order_seasonal_ar), k_seasonal_diff,
-                               str(order_seasonal_ma), self.model.seasonal_periods))
+                               str(order_seasonal_ma),
+                               self.model.seasonal_periods))
             if not order == '':
                 order += 'x'
         model_name = (
